// -*- C++ -*-
/*!
 * @file InPortConnector.h
 * @brief InPortConnector base class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2009-2010
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef RTC_INPORTCONNECTOR_H
#define RTC_INPORTCONNECTOR_H

#include <rtm/ConnectorBase.h>

namespace RTC
{
  /*!
   * @if jp
   * @class InPortConnector
   * @brief InPortConnector NX
   *
   * InPort  Push/Pull e Connector h邽߂̊NXB
   *
   * @since 1.0.0
   *
   * @else
   * @class InPortConnector
   * @brief InPortConnector base class
   *
   * The base class to derive subclasses for InPort's Push/Pull Connectors
   *
   * @since 1.0.0
   *
   * @endif
   */
  class InPortConnector
    : public ConnectorBase
  {
  public:
    DATAPORTSTATUS_ENUM
    /*!
     * @if jp
     * @brief RXgN^
     *
     * @param info ڑ܂ ConnectorInfo IuWFNg
     * @param buffer ̃RlN^̃obt@ւ̃|C^
     *
     * @else
     * @brief Constructor
     *
     * @param info ConnectorInfo object which includes connection information
     * @param buffer A pointer to the buffer of the connector
     *
     * @endif
     */
    InPortConnector(ConnectorInfo& info,
                    CdrBufferBase* buffer);

    /*!
     * @if jp
     * @brief fXgN^
     * @else
     * @brief Destructor
     * @endif
     */
    virtual ~InPortConnector();

   /*!
     * @if jp
     * @brief ConnectorInfo 擾
     *
     * Connector ConnectorInfo 擾
     *
     * @return ̃RlN^ێ ConnectorInfo IuWFNg
     *
     * @else
     * @brief Getting ConnectorInfo
     *
     * This operation returns ConnectorInfo
     *
     * @return ConnectorInfo object which is owned by this connector
     *
     * @endif
     */
    virtual const ConnectorInfo& profile();

    /*!
     * @if jp
     * @brief Connector ID 擾
     *
     * Connector ID 擾
     *
     * @return RlN^ ID ւ̃|C^
     *
     * @else
     * @brief Getting Connector ID
     *
     * This operation returns Connector ID
     *
     * @return A pointer to the connector id string
     *
     * @endif
     */
    virtual const char* id();

    /*!
     * @if jp
     * @brief Connector 擾
     *
     * Connector 擾
     *
     * @return RlN^ւ̃|C^
     *
     * @else
     * @brief Getting Connector name
     *
     * This operation returns Connector name
     *
     * @return A pointer to the connector's name string
     *
     * @endif
     */
    virtual const char* name();

    /*!
     * @if jp
     * @brief ڑ֐
     *
     * Connector ێĂڑ
     *
     * @return ReturnCode
     *
     * @else
     * @brief Disconnect connection
     *
     * This operation disconnect this connection
     *
     * @return ReturnCode
     *
     * @endif
     */
    virtual ReturnCode disconnect() = 0;

    /*!
     * @if jp
     * @brief Buffer 擾
     *
     * Connector ێĂ Buffer Ԃ
     *
     * @return ̃RlN^ێobt@ւ̃|C^
     *
     * @else
     * @brief Getting Buffer
     *
     * This operation returns this connector's buffer
     *
     * @return A pointer to the buffer owned by this connector
     *
     * @endif
     */
    virtual CdrBufferBase* getBuffer();

    /*!
     * @if jp
     * @brief read ֐
     *
     * Buffer f[^ InPort  read ֐
     *
     * @param data ̃RlN^ǂݏof[^i[ϐւ̎Q
     * @return ReturnCode
     *
     * @else
     * @brief Destructor
     *
     * The read function to read data from buffer to InPort
     *
     * @param data A reference to a variable to which data from this
     *             connector is stored.
     * @return ReturnCode
     *
     * @endif
     */
    virtual ReturnCode read(cdrMemoryStream& data) = 0;

    /*!
     * @if jp
     * @brief endian^Cvݒ
     *
     * endian^Cvݒ肷
     *
     * @param endian_type true: little, false: big
     *
     * @else
     * @brief Setting an endian type
     *
     * This operation set this connector's endian type
     *
     * @param endian_type true: little, false: big
     *
     * @endif
     */
    virtual void setEndian(const bool endian_type);

    /*!
     * @if jp
     * @brief endian ݒԂ
     *
     * ̃RlN^ɐݒ肳ꂽGfBA little endian ǂB
     *
     * @return true: little endian, false: big endian
     *
     * @else
     * @brief Whether this connector's endian is little
     *
     * This operation returns whether the connector's endian is little or not.
     *
     * @return true: little endian, false: big endian
     *
     * @endif
     */
    virtual bool isLittleEndian();

  protected:
    /*!
     * @if jp
     * @brief K[Xg[
     * @else
     * @brief Logger stream
     * @endif
     */
    Logger rtclog;
    /*!
     * @if jp
     * @brief ConnectorInfo
     * @else
     * @brief ConnectorInfo 
     * @endif
     */
    ConnectorInfo m_profile;
    /*!
     * @if jp
     * @brief Connector ێĂ Buffer
     * @else
     * @brief Connector's buffer
     * @endif
     */
    CdrBufferBase* m_buffer;
    /*!
     * @if jp
     * @brief ڑGfBA 
     * @else
     * @brief Connected Endian
     * @endif
     */
    bool m_littleEndian;
  };
}; // namespace RTC

#endif // RTC_INPORTCONNECTOR_H
