// -*- C++ -*-
/*!
 * @file  OutPortConsumer.h
 * @brief OutPortConsumer class
 * @date  $Date: 2007-12-31 03:08:05 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: OutPortConsumer.h 2151 2011-05-31 00:23:51Z n-ando $
 *
 */

#ifndef RTC_OUTPORTCONSUMER_H
#define RTC_OUTPORTCONSUMER_H

#include <coil/Factory.h>
#include <rtm/DataPortStatus.h>
#include <rtm/CdrBufferBase.h>

// Why RtORB does not allow the following foward declaration?
#ifndef ORB_IS_RTORB
namespace SDOPackage
{
  class NVList;
};
#endif // ORB_IS_RTORB

namespace coil
{
  class Properties;
};

namespace RTC
{
  class ConnectorListeners;
  class ConnectorInfo;

  /*!
   * @if jp
   *
   * @class OutPortConsumer
   *
   * @brief OutPortConsumer ۃNX
   *
   * OutPort  REQUIRED C^[tF[X邽߂̒ۊNXB
   * ̃TuNX̃IuWFNg InPort ɑApull ^̃f[^Xg
   * [BInPort ɑ΂ĐVC^[tF[X
   * ꍇɂ́ÃNXpAȉ̊֐KvB
   * 
   * - init()
   * - setBuffer()
   * - setListener()
   * - get()
   *
   * ɁAȉ̉z֐ɁAConnectorProfile KvƂ
   * ȂǁAڑm邢͐ڑ̐ؒfs邽߂ɕKvȏ
   * Ȃ΂ȂȂB
   *
   * - subscribeInterface()
   * - unsubscribeInterface()
   *
   * InPort  OutPortConsumer ̃t@NgǗNXɑ΂ėp\
   *  OutPortConsumer ⍇A񋟉\ȃC^[tF[X^CvO
   * ɐ錾B]āAInPort@ɑ΂ REQUIRED C^[tF[X
   * 񋟂 OutPortConsumer ̃TuNX́AOutPortConsumerFactory
   * Ƀt@Ng֐o^KvB
   *
   * RTC::OutPortConsumerFactory::instance().addFactory() A
   *
   * - 1: voC_̖O, "corba_cdr" Ȃ
   * - 2: t@Ng֐ coil::Creator<B, T>
   * - 3: 폜֐ coil::Destructor<B, T>
   * 
   * ^ČĂяoKvBȉ́At@Ngւ̓o^ƁA
   * ֐ƂłB
   * 
   * <pre>
   * extern "C"
   * {
   *   void OutPortCorbaCdrConsumerInit(void)
   *   {
   *     RTC::OutPortConsumerFactory&
   *                         factory(RTC::OutPortConsumerFactory::instance());
   *     factory.addFactory("corba_cdr",
   *                        ::coil::Creator<::RTC::OutPortConsumer,
   *                                        ::RTC::OutPortCorbaCdrConsumer>,
   *                        ::coil::Destructor<::RTC::OutPortConsumer,
   *                                           ::RTC::OutPortCorbaCdrConsumer>);
   *   }
   * };
   * </pre>
   *
   * ̗̂悤ɁAt@Ngւ̓o^֐ƂāAextern "C"
   * ɂV{QƉ\ɂĂB邱ƂŁA
   * OutPortConsumer LIuWFNg (DLL) ēI[h\
   * AvoC_̌^𓮓Iɒǉ邱Ƃ\ƂȂB
   *
   * @since 0.4.0
   *
   * @else
   * @class OutPortConsumer
   *
   * @brief OutPortConsumer abstract class
   *
   * The virtual class for OutPort's PROVIDED interface
   * implementation.  New interface for OutPort have to inherit this
   * class, and have to implement the following functions.
   *
   * - init()
   * - setBuffer()
   * - setListener()
   * - get()
   *
   * Furthermore, connecting or disconnecting processes, such as
   * obtaining some information from ConnectorProfile or releasing
   * some resources, should be implemented in the following virtual
   * functions.
   *
   * - subscribeInterface()
   * - unsubscribeInterface()
   *
   * InPort inquires available OutPortConsumers to the factory class
   * of OutPortConsumer, and publishes available interfaces to
   * others. Therefore, sub-classes of OutPortConsumer that provides
   * PROVIDED interface to OutPort should register its factory to
   * OutPortConsumerFactory.
   *
   * RTC::OutPortConsumerFactory::instance().addFactory() would be
   * called with the following arguments.
   *
   * 1st arg: The name of provider. ex. "corba_cdr"
   * 2nd arg: Factory function. coil::Creator<B, T>
   * 3rd arg: Destruction function. coil::Destructor<B, T>
   *
   * The following example shows how to register factory function.
   * And it is also declared as a initialization function.
   *
   * <pre>
   * extern "C"
   * {
   *   void OutPortCorbaCdrConsumerInit(void)
   *   {
   *     RTC::OutPortConsumerFactory&
   *                         factory(RTC::OutPortConsumerFactory::instance());
   *     factory.addFactory("corba_cdr",
   *                        ::coil::Creator<::RTC::OutPortConsumer,
   *                                        ::RTC::OutPortCorbaCdrConsumer>,
   *                        ::coil::Destructor<::RTC::OutPortConsumer,
   *                                           ::RTC::OutPortCorbaCdrConsumer>);
   *   }
   * };
   * </pre>
   *
   * It is recommended that the registration process is declared as a
   * initialization function with "extern C" to be accessed from the
   * outside of module.  If the OutPortConsumers are compiled as a
   * shared object or DLL for dynamic loading, new OutPortConsumer
   * types can be added dynamically.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class OutPortConsumer
    : public DataPortStatus
  {
  public:
    DATAPORTSTATUS_ENUM
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * zfXgN^B
     *
     * @else
     * @brief Destructor
     *
     * Virtual destructor
     *
     * @endif
     */
    virtual ~OutPortConsumer(void){};

    /*!
     * @if jp
     * @brief ݒ菉
     *
     * OutPortConsumer̊eݒsBNXł́A^ꂽ
     * PropertiesKvȏ擾ĊeݒsB init() 
     * ́AOutPortProviderエсAڑɂꂼĂ΂
     * \BāÅ֐͕Ă΂邱Ƃz肵ċL
     * qׂłB
     * 
     * @param prop ݒ
     *
     * @else
     *
     * @brief Initializing configuration
     *
     * This operation would be called to configure in initialization.
     * In the concrete class, configuration should be performed
     * getting appropriate information from the given Properties data.
     * This function might be called right after instantiation and
     * connection sequence respectivly.  Therefore, this function
     * should be implemented assuming multiple call.
     *
     * @param prop Configuration information
     *
     * @endif
     */
    virtual void init(coil::Properties& prop) = 0;

    /*!
     * @if jp
     * @brief obt@Zbg
     *
     * OutPortConsumerf[^oobt@ZbgB
     * łɃZbgꂽobt@ꍇAȑÕobt@ւ
     * |C^ɑ΂ď㏑B
     * OutPortProvider̓obt@̏L肵ĂȂ̂ŁA
     * obt@̍폜̓[U̐ӔCōsȂ΂ȂȂB
     *
     * @param buffer OutPortProviderf[^oobt@ւ̃|C^
     *
     * @else
     * @brief Setting outside buffer's pointer
     *
     * A pointer to a buffer from which OutPortProvider retrieve data.
     * If already buffer is set, previous buffer's pointer will be
     * overwritten by the given pointer to a buffer.  Since
     * OutPortProvider does not assume ownership of the buffer
     * pointer, destructor of the buffer should be done by user.
     * 
     * @param buffer A pointer to a data buffer to be used by OutPortProvider
     *
     * @endif
     */
    virtual void setBuffer(CdrBufferBase* buffer) = 0;

    /*!
     * @if jp
     * @brief Xiݒ肷B
     *
     * OutPort ̓f[^MɂeCxgɑ΂ē̃Xi
     * IuWFNgR[R[obN@\񋟂Bڍׂ
     * ConnectorListener.h  ConnectorDataListener, ConnectorListener
     * QƂ̂ƁBOutPortProvider ̃TuNXł́A^ꂽX
     * iK؂ȃ^C~OŌĂяoׂłBAׂẴXi
     * ĂяoKv͂ȂB
     *
     * @param info ڑ
     * @param listeners XiIuWFNg
     *
     * @else
     * @brief Set the listener. 
     *
     * OutPort provides callback functionality that calls specific
     * listener objects according to the events in the data publishing
     * process. For details, see documentation of
     * ConnectorDataListener class and ConnectorListener class in
     * ConnectorListener.h. In the sub-classes of OutPortProvider, the
     * given listeners should be called in the proper timing. However,
     * it is not necessary to call all the listeners.
     *
     * @param info Connector information
     * @param listeners Listener objects
     *
     * @endif
     */
    virtual void setListener(ConnectorInfo& info,
                             ConnectorListeners* listeners) = 0;

    /*!
     * @if jp
     *
     * @brief f[^M
     *
     * f[^Ms邽߂̏z֐B
     * ۃNXł́Aꂼ̕@Ń[gOutPortf[^
     * M郍WbNB
     * MɊւԂɉĈȉ̖߂lԂB
     *
     * @param data Mf[^
     * @return PORT_OK         I
     *         BUFFER_TIMEOUT  ^CAEg
     *         RECV_EMPTY      擾̃obt@łB
     *         CONNECTION_LOST ڑؒfꂽ
     *         PORT_ERROR      G[
     *         UNKNOWN_ERROR   {肦ȂG[
     *
     * @else
     *
     * @brief Receive data
     *
     * Pure virtual function to receive data.
     *
     * @endif
     */
    virtual ReturnCode get(cdrMemoryStream& data) = 0;

    /*!
     * @if jp
     *
     * @brief f[^Mʒmւ̓o^
     *
     * w肳ꂽvpeBɊÂāAf[^Mʒmɓo^邽߂
     * z֐B
     *
     * @param properties o^pvpeB
     *
     * @return o^(o^:trueAo^s:false)
     *
     * @else
     *
     * @brief Subscribe the data receive notification
     *
     * Pure virtual function to subscribe the data receive notification
     * based on specified property information.
     *
     * @param properties Properties for subscription
     *
     * @return Subscription result (Successful:true, Failed:false)
     *
     * @endif
     */
    virtual bool subscribeInterface(const SDOPackage::NVList& properties) = 0;
    
    /*!
     * @if jp
     *
     * @brief f[^Mʒm̓o^
     *
     * f[^Mʒm̓o^邽߂̏z֐B
     *
     * @param properties o^pvpeB
     *
     * @return o^(o^:trueAo^s:false)
     *
     * @else
     *
     * @brief Unsubscribe the data receive notification
     *
     * Pure virtual function to unsubscribe the data receive notification.
     *
     * @param properties Properties for unsubscription
     *
     * @return Unsubscription result (Successful:true, Failed:false)
     *
     * @endif
     */
    virtual void unsubscribeInterface(const SDOPackage::NVList& properties) = 0;

  protected:
    /*!
     * @if jp
     * @brief K[Xg[
     * @else
     * @brief Logger stream
     * @endif
     */
    mutable Logger rtclog;

    /*!
     * @if jp
     * @brief InterfaceڑpFunctor
     * @else
     * @brief Functor to subscribe the interface
     * @endif
     */
    struct subscribe
    {
      subscribe(const SDOPackage::NVList& prop) : m_prop(prop) {}
      void operator()(OutPortConsumer* consumer)
      {
        consumer->subscribeInterface(m_prop);
      }
      const SDOPackage::NVList& m_prop;
    };
    
    /*!
     * @if jp
     * @brief InterfaceڑpFunctor
     * @else
     * @brief Functor to unsubscribe the interface
     * @endif
     */
    struct unsubscribe
    {
      unsubscribe(const SDOPackage::NVList& prop) : m_prop(prop) {}
      void operator()(OutPortConsumer* consumer)
      {
        consumer->unsubscribeInterface(m_prop);
      }
      const SDOPackage::NVList& m_prop;
    };
  };

  /*!
   * @if jp
   * @brief OutPortConsumerFactory^錾
   * @else
   * @brief OutPortConsumerFactory type definition
   * @endif
   */
  typedef ::coil::GlobalFactory<OutPortConsumer> OutPortConsumerFactory;

#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)
  EXTERN template class DLL_PLUGIN ::coil::GlobalFactory<OutPortConsumer>;
#endif
};     // namespace RTC
#endif // RTC_OUTPORTCONSUMER_H

